from mock import patch
from tornado.concurrent import Future
from tornado.httpclient import HTTPRequest
from tornado.testing import AsyncHTTPTestCase, gen_test
from tornado.web import Application
from tornado.websocket import websocket_connect

from tornwamp import messages, session, topic
from tornwamp.handler import WAMPHandler, deliver_messages
from tornwamp.processors.pubsub import customize as pubsub_customize


class MockWebsocket(object):
    message = None

    def write_message(self, msg):
        self.message = msg


class MockConnection(object):

    def __init__(self):
        self._websocket = MockWebsocket()

    def add_subscription_channel(self, *args):
        pass


class MockMessage(object):
    json = "mocked message"


class UnauthorizeWAMPHandler(WAMPHandler):

    def authorize(self):
        return False, {}, "Denied"


class WAMPHandlerTestCase(AsyncHTTPTestCase):

    def setUp(self):
        session.connections = {}
        super(WAMPHandlerTestCase, self).setUp()

    def get_app(self):
        application = Application([
            (r"/ws", WAMPHandler),
            (r"/no", UnauthorizeWAMPHandler)
        ])
        return application

    def test_deliver_messages(self):
        websocket = MockWebsocket()
        items = [
            {
                "websocket": websocket,
                "message": MockMessage()
            }
        ]
        deliver_messages(items)
        self.assertEqual(websocket.message, "mocked message")

    def build_request(self, path="ws", headers=None):
        port = self.get_http_port()
        url = 'ws://0.0.0.0:{0}/{1}'.format(port, path)
        if not headers:
            headers = {}
        if 'Origin' not in headers:
            headers['Origin'] = 'http://0.0.0.0:%d' % port
        headers['Sec-WebSocket-Protocol'] = 'wamp.2.json'
        return HTTPRequest(url, headers=headers)

    @gen_test
    def test_connection_succeeds(self):
        request = self.build_request()
        ws = yield websocket_connect(request)
        msg = messages.HelloMessage(realm="burger.friday")
        ws.write_message(msg.json)

        response = yield ws.read_message()
        message = messages.Message.from_text(response)
        self.assertIs(message.code, messages.Code.WELCOME)
        self.assertFalse(getattr(ws, "close_code", False))
        ws.close()

    @gen_test
    def test_connection_with_xforwardedfor(self):
        request = self.build_request(headers={"X-Forwarded-For": "10.0.0.1"})
        ws = yield websocket_connect(request)
        msg = messages.HelloMessage(realm="burger.thursday")
        ws.write_message(msg.json)

        connection_ip = list(session.connections.values())[0]
        self.assertTrue(connection_ip.peer.startswith("10.0.0.1"))

        response = yield ws.read_message()
        message = messages.Message.from_text(response)
        self.assertIs(message.code, messages.Code.WELCOME)
        self.assertFalse(getattr(ws, "close_code", False))
        ws.close()

    @gen_test
    def test_authorization_fails(self):
        request = self.build_request(path="no")
        ws = yield websocket_connect(request)
        text = yield ws.read_message()

        message = messages.AbortMessage.from_text(text)
        self.assertIs(message.code, messages.Code.ABORT)
        self.assertEqual(message.reason, 'tornwamp.error.unauthorized')
        self.assertEqual(message.details['message'], "Denied")

        msg = yield ws.read_message()
        self.assertIs(msg, None)
        self.assertEqual(ws.close_code, 1)
        self.assertEqual(ws.close_reason, "Denied")

    @gen_test
    def test_goodbye_message_closes_connection(self):
        request = self.build_request()
        ws = yield websocket_connect(request)

        msg = messages.GoodbyeMessage(details={"message": "Closing for test purposes"}, reason="close.up")
        ws.write_message(msg.json)

        text = yield ws.read_message()
        message = messages.GoodbyeMessage.from_text(text)
        self.assertIs(message.code, messages.Code.GOODBYE)
        self.assertEqual(message.reason, 'close.up')
        self.assertEqual(message.details['message'], "Closing for test purposes")

        msg = yield ws.read_message()
        self.assertIs(msg, None)
        self.assertEqual(ws.close_code, 1000)
        self.assertEqual(ws.close_reason, "Closing for test purposes")

    @gen_test
    @patch("tornwamp.processors.pubsub.customize.authorize_subscription", return_value=(True, ""))
    def test_publish_event(self, mock_authorize):
        connection_subscriber = MockConnection()
        topic.topics.add_subscriber("interstellar", connection_subscriber)

        request = self.build_request()
        ws = yield websocket_connect(request)

        options = {"acknowledge": True}
        msg = messages.PublishMessage(request_id=1, topic="interstellar", kwargs={"review": "Mind blowing"}, options=options)
        ws.write_message(msg.json)

        text = yield ws.read_message()
        published_msg = messages.PublishedMessage.from_text(text)
        self.assertEqual(published_msg.request_id, 1)

        text = connection_subscriber._websocket.message
        event_msg = messages.EventMessage.from_text(text)
        self.assertEqual(event_msg.kwargs, {u'review': u'Mind blowing'})



"""
TornWAMP user-configurable structures.
"""
from tornwamp.processors import GoodbyeProcessor, HelloProcessor, pubsub, rpc
from tornwamp.messages import Code

processors = {
    Code.HELLO: HelloProcessor,
    Code.GOODBYE: GoodbyeProcessor,
    Code.SUBSCRIBE: pubsub.SubscribeProcessor,
    Code.CALL: rpc.CallProcessor,
    Code.PUBLISH: pubsub.PublishProcessor
}
#    2: 'welcome',
#    3: 'abort',
#    4: 'challenge',
#    5: 'authenticate',
#    7: 'heartbeat',
#    8: 'error',
#    17: 'published',
#    33: 'subscribed',
#    34: 'unsubscribe',
#    35: 'unsubscribed',
#    36: 'event',
#    49: 'cancel',
#    50: 'result',
#    64: 'register',
#    65: 'registered',
#    66: 'unregister',
#    67: 'unregistered',
#    68: 'invocation',
#    69: 'interrupt',
#    70: 'yield'



import unittest
from datetime import datetime

from mock import patch
from tornwamp.session import ClientConnection, ConnectionDict


class ClientConnectionTestCase(unittest.TestCase):

    @patch("tornwamp.session.datetime")
    @patch("tornwamp.session.create_global_id", return_value=1111)
    def test_client_connection_with_details(self, mock_global_id, mock_datetime):
        mock_datetime.now.return_value = datetime(1984, 5, 11)
        connection = ClientConnection(websocket=None, user_name="Alex", speaks_chinese=True)
        self.assertEqual(connection.user_name, "Alex")
        self.assertTrue(connection.speaks_chinese)
        expected_response = {
            "user_name": "Alex",
            "speaks_chinese": True,
            "id": 1111,
            "topics": {'subscriber': {}, 'publisher': {}},
            "zombie": False,
            'zombification_datetime': None,
            "last_update": '1984-05-11T00:00:00'
        }
        self.assertEqual(connection.dict, expected_response)

    @patch("tornwamp.session.datetime")
    def test_connection_zombify(self, mock_datetime):
        mock_datetime.now.return_value = datetime(2002, 4, 4)
        connection = ClientConnection(websocket=None)
        self.assertFalse(connection.zombie)
        connection.zombify()
        self.assertEqual(connection.zombification_datetime, '2002-04-04T00:00:00')
        self.assertEqual(connection.topics, {'subscriber': {}, 'publisher': {}})
        self.assertTrue(connection.zombie)

    def test_add_subscription_channel(self):
        connection = ClientConnection(websocket=None)
        connection.add_subscription_channel(7, "start.wars")
        expected_topics = {
            "subscriber": {
                "start.wars": 7
            },
            'publisher': {}
        }
        self.assertEqual(connection.topics, expected_topics)

    def test_get_publisher_topics(self):
        connection = ClientConnection(websocket=None)
        connection.add_subscription_channel(12, "again.do.nothing")
        connection.add_subscription_channel(32, "start.wars")
        expected_topics = ["again.do.nothing", "start.wars"]
        topics = connection.get_subscriber_topics()
        self.assertEqual(sorted(topics), expected_topics)

    def test_get_subscription_id(self):
        connection = ClientConnection(websocket=None)
        connection.add_subscription_channel(5, "a.new.hope")
        connection.add_subscription_channel(6, "the.empire.strikes.back")
        subscription_id = connection.get_subscription_id("a.new.hope")
        self.assertEqual(subscription_id, 5)

    def test_add_publishing_channel(self):
        connection = ClientConnection(websocket=None)
        connection.add_publishing_channel(42, "reason.for.life")
        expected_topics = {
            "publisher": {
                "reason.for.life": 42
            },
            'subscriber': {}
        }
        self.assertEqual(connection.topics, expected_topics)

    def test_get_publisher_topics(self):
        connection = ClientConnection(websocket=None)
        connection.add_publishing_channel(11, "dont.try.anything")
        connection.add_publishing_channel(42, "reason.for.life")
        expected_topics = ["dont.try.anything", "reason.for.life"]
        topics = connection.get_publisher_topics()
        self.assertEqual(sorted(topics), expected_topics)

    def test_get_topics(self):
        connection = ClientConnection(websocket=None)
        connection.add_subscription_channel(12, "whiplash")
        connection.add_publishing_channel(21, "reload")
        topics = connection.get_topics()
        expected_topics = {'reload': 21, 'whiplash': 12}
        self.assertEqual(topics, expected_topics)


class ConnectionDicttestCase(unittest.TestCase):

    @patch("tornwamp.session.datetime")
    @patch("tornwamp.session.create_global_id", return_value=2222)
    def test_connection_dict(self, mock_global_id, mock_datetime):
        mock_datetime.now.return_value = datetime(1950, 4, 6)
        connections = ConnectionDict()
        connections[2222] = ClientConnection(websocket=None, include=1, _exclude=True)
        expected_response = {
            2222: {
                "include": 1,
                "id": 2222,
                "topics": {'subscriber': {}, 'publisher': {}},
                "zombie": False,
                'zombification_datetime': None,
                "last_update": '1950-04-06T00:00:00'
            }
        }
        self.assertEqual(connections.dict, expected_response)

    def test_filter_by_property_value(self):
        connections = ConnectionDict()
        connections[1] = ClientConnection(websocket=None, name="Mario")
        connections[2] = ClientConnection(websocket=None, name="Goncalo")
        connections[3] = ClientConnection(websocket=None, name="Fredrik")
        answer = connections.filter_by_property_value("name", "Mario")
        self.assertEqual(answer, [connections[1]])



"""
Abstract websocket connections (dual channel between clients and server).
"""
from datetime import datetime

from tornwamp import topic
from tornwamp.identifier import create_global_id


class ConnectionDict(dict):
    """
    Connections manager.
    """

    @property
    def dict(self):
        """
        Return a python dictionary which could be jsonified.
        """
        return {key: value.dict for key, value in self.items()}

    def filter_by_property_value(self, attr_name, attr_value):
        """
        Provided an attribute name and its value, retrieve connections which
        have it.
        """
        items = []
        for _, connection in self.items():
            if getattr(connection, attr_name) == attr_value:
                items.append(connection)
        return items


connections = ConnectionDict()


class ClientConnection(object):
    """
    Represent a client connection.
    """
    existing_ids = []

    def __init__(self, websocket, **details):
        """
        Create a connection object provided:
        - websocket (tornado.websocket.WebSocketHandler instance
        - details: dictionary of metadata associated to the connection
        """
        self.id = create_global_id()

        # set connection attributes, if any is given
        for name, value in details.items():
            setattr(self, name, value)

        # meta-data
        # TODO: update this
        self.last_update = datetime.now().isoformat()

        # communication-related
        self._websocket = websocket
        self.topics = {
            "subscriber": {},
            "publisher": {}
        }

        # when connection should be closed but something is left
        self.zombie = False
        self.zombification_datetime = None

    @property
    def peer(self):
        try:
            ip, port = self._websocket.ws_connection.stream.socket.getpeername()
        except AttributeError:
            ip = self._websocket.request.remote_ip
            name = u"{0}:HACK|{1}".format(ip, self.id)
        else:
            forwarded_ip = self._websocket.request.headers.get("X-Forwarded-For")
            if forwarded_ip:
                ip = forwarded_ip
            name = u"{0}:{1}|{2}".format(ip, port, self.id)
        return name

    def get_subscription_id(self, topic_name):
        """
        Return connection's subscription_id for a specific topic.
        """
        subscribe_subscription = self.topics['subscriber'].get(topic_name)
        publish_subscription = self.topics['publisher'].get(topic_name)
        return subscribe_subscription or publish_subscription

    def add_subscription_channel(self, subscription_id, topic_name):
        """
        Add topic as a subscriber.
        """
        self.topics["subscriber"][topic_name] = subscription_id

    def remove_subscription_channel(self, topic_name):
        """
        Remove topic as a subscriber.
        """
        self.topics.get("subscriber", {}).pop(topic_name, None)

    def add_publishing_channel(self, subscription_id, topic_name):
        """
        Add topic as a publisher.
        """
        self.topics["publisher"][topic_name] = subscription_id

    def remove_publishing_channel(self, topic_name):
        """
        Remove topic as a publisher.
        """
        self.topics.get("publisher", {}).pop(topic_name, None)

    def get_publisher_topics(self):
        """
        Return list of topics to which this connection has subscribed.
        """
        return list(self.topics["publisher"])

    def get_topics(self):
        """
        Return a dictionary containing subscriptions_ids and connections - no
        matter if they are subscribers or publishers.
        """
        return dict(self.topics["subscriber"], **self.topics["publisher"])

    @property
    def topics_by_subscription_id(self):
        return {subscription_id: topic for topic, subscription_id in self.get_topics().items()}

    @property
    def dict(self):
        """
        Return dict representation of the current Connection, keeping only data
        that could be exported to JSON (convention: attributes which do not
        start with _).
        """
        return {k: v for k, v in self.__dict__.items() if not k.startswith('_')}

    def zombify(self):
        """
        Make current connection a zombie:
        - remove all its topics
        - remove it from the TopicsManager

        In WAMP, in order to disconnect, we're supposed to do a GOODBYE
        handshake.

        Considering the server wanted to disconnect the client for some reason,
        we leave the client in a "zombie" state, so it can't subscribe to
        topics and can't receive messages from other clients.
        """
        self.zombification_datetime = datetime.now().isoformat()
        self.zombie = True
        topic.topics.remove_connection(self)



"""
Implement Tornado WAMP Handler.
"""
from tornado.websocket import WebSocketHandler

from tornwamp import customize, session
from tornwamp.messages import AbortMessage, Message
from tornwamp.processors import UnhandledProcessor


SUBPROTOCOL = 'wamp.2.json'


def abort(handler, error_msg, details, reason='tornwamp.error.unauthorized'):
    """
    Used to abort a connection while the user is trying to establish it.
    """
    abort_message = AbortMessage(reason=reason)
    abort_message.error(error_msg, details)
    handler.write_message(abort_message.json)
    handler.close(1, error_msg)


def deliver_messages(items):
    """
    Receives a dictionary with {websocket, message} and writes the message into the websocket
    """
    for item in items:
        recipient = item["websocket"]
        recipient.write_message(item["message"].json)


class WAMPHandler(WebSocketHandler):
    """
    WAMP WebSocket Handler.
    """

    def __init__(self, *args, **kargs):
        self.connection = None
        super(WAMPHandler, self).__init__(*args, **kargs)

    def select_subprotocol(self, subprotocols):
        "Select WAMP 2 subprocotol"
        return SUBPROTOCOL

    def authorize(self):
        """
        Override for authorizing connection before the WebSocket is opened.
        Sample usage: analyze the request cookies.

        Return a tuple containing:
        - boolean (if connection was accepted or not)
        - dict (containing details of the authentication)
        - string (explaining why the connection was not accepted)
        """
        return True, {}, ""

    def register_connection(self):
        """
        Add connection to connection's manager.
        """
        session.connections[self.connection.id] = self.connection

    def deregister_connection(self):
        """
        Remove connection from connection's manager.
        """
        return session.connections.pop(self.connection.id, None) if self.connection else None

    def open(self):
        """
        Responsible for authorizing or aborting WebSocket connection.
        It calls 'authorize' method and, based on its response, sends
        a ABORT message to the client.
        """
        authorized, details, error_msg = self.authorize()
        if authorized:
            self.connection = session.ClientConnection(self, **details)
            self.register_connection()
        else:
            abort(self, error_msg, details)

    def on_message(self, txt):
        """
        Handle incoming messages on the WebSocket. Each message will be parsed
        and handled by a Processor, which can be (re)defined by the user
        changing the value of 'processors' dict, available at
        tornwamp.customize module.
        """
        msg = Message.from_text(txt)
        Processor = customize.processors.get(msg.code, UnhandledProcessor)
        processor = Processor(msg, self.connection)

        if self.connection and not self.connection.zombie:  # TODO: cover branch else
            if processor.answer_message is not None:
                self.write_message(processor.answer_message.json)

        deliver_messages(processor.direct_messages)

        if processor.must_close:
            self.close(processor.close_code, processor.close_reason)

    def close(self, code=None, reason=None):
        """
        Invoked when a WebSocket is closed.
        """
        self.deregister_connection()
        super(WAMPHandler, self).close(code, reason)



"""
Processors are responsible for parsing received WAMP messages and providing
feedback to the server on what should be done (e.g. send answer message order
close connection).
"""
import abc
import six
from tornwamp.messages import Message, ErrorMessage, GoodbyeMessage, HelloMessage, WelcomeMessage


class Processor(six.with_metaclass(abc.ABCMeta)):
    """
    Abstract class which defines the base behavior for processing messages
    sent to the Websocket.

    Classes that extend this are supposed to overwride process method.
    """
    __metaclass__ = abc.ABCMeta

    def __init__(self, message, connection):
        """
        msg: json
        session_id: string or number
        """
        self.session_id = getattr(connection, "id", None)
        self.connection = connection

        # message just received by the WebSocket
        self.message = message

        # messages to be sent by the WebSocket
        self.answer_message = None  # response message

        self.direct_messages = []  # to send direct messages to other connections
        # each item must have at least two keys: "connection" and "message"

        # the attributes below are in case we are expected to close the socket
        self.must_close = False
        self.close_code = None
        self.close_reason = None

        self.process()

    @abc.abstractmethod
    def process(self):
        """
        Responsible for processing the input message and may change the default
        values for the following attributes:
        - answer_message
        - broadcast_message
        - group_messages

        - must_close
        - close_code (1000 or in the range 3000 to 4999)
        - close_message
        """


class UnhandledProcessor(Processor):
    """
    Raises an error when the provided message can't be parsed
    """

    def process(self):
        message = Message(*self.message.value)
        description = "Unsupported message {0}".format(self.message.value)
        out_message = ErrorMessage(
            request_code=message.code,
            request_id=message.id,
            uri='wamp.unsupported.message'
        )
        out_message.error(description)
        self.answer_message = out_message


class HelloProcessor(Processor):
    """
    Responsible for dealing HELLO messages.
    """
    def process(self):
        """
        Return WELCOME message based on the input HELLO message.
        """
        # hello_message = HelloMessage(*self.message.value)
        # TODO: assert realm is in allowed list
        welcome_message = WelcomeMessage()
        self.answer_message = welcome_message


class GoodbyeProcessor(Processor):
    """
    Responsible for dealing GOODBYE messages.
    """
    def process(self):
        self.answer_message = GoodbyeMessage(*self.message.value)
        self.must_close = True
        # Excerpt from RFC6455 (The WebSocket Protocol)
        # "Endpoints MAY: use the following pre-defined status codes when sending
        # a Close frame:
        #   1000 indicates a normal closure, meaning that the purpose for
        #   which the connection was established has been fulfilled."
        # http://tools.ietf.org/html/rfc6455#section-7.4
        self.close_code = 1000
        self.close_reason = self.answer_message.details.get('message', '')



import unittest
from mock import patch

from tornwamp import identifier


class IdentifierTestCase(unittest.TestCase):

    def setUp(self):
        identifier.existing_ids = []

    def tearDown(self):
        identifier.existing_ids = []

    @patch("tornwamp.identifier.random.randint", return_value=10)
    def test_create_global_id(self, randint):
        self.assertEqual(identifier.existing_ids, [])
        new_id = identifier.create_global_id()
        self.assertEqual(new_id, 10)
        self.assertEqual(identifier.existing_ids, [10])
        self.assertEqual(randint.call_count, 1)

    @patch("tornwamp.identifier.random.randint", side_effect=[1105184, 604950])
    def test_create_global_id_random_hit(self, randint):
        identifier.existing_ids = [1105184]
        new_id = identifier.create_global_id()
        self.assertEqual(new_id, 604950)
        self.assertEqual(identifier.existing_ids, [1105184, 604950])
        self.assertEqual(randint.call_count, 2)



import unittest

from mock import patch
from tornwamp.topic import Topic, TopicsManager
from tornwamp.session import ClientConnection


class MockSubscriber(object):
    dict = {"author": "plato"}


class TopicTestCase(unittest.TestCase):

    def test_constructor(self):
        topic = Topic("the.monarchy")
        self.assertEqual(topic.name, "the.monarchy")
        self.assertEqual(topic.subscribers, {})
        self.assertEqual(topic.publishers, {})

    def test_dict(self):
        topic = Topic("the.republic")
        subscriber = MockSubscriber()
        topic.subscribers[123] = subscriber
        expected_dict = {
            'name': 'the.republic',
            'publishers': {},
            'subscribers': {
                123: {"author": "plato"}
            }
        }
        self.assertEqual(topic.dict, expected_dict)

    def test_connections(self):
        topic = Topic("start.trek")
        topic.subscribers = {1: 2}
        topic.publishers = {3: 4}
        expected = {1: 2, 3: 4}
        self.assertEqual(topic.connections, expected)


class TopicsManagerTestCase(unittest.TestCase):

    maxDiff = None

    def test_add_subscriber(self):
        manager = TopicsManager()
        connection = ClientConnection(None, name="Dracula")
        manager.add_subscriber("romania", connection, 432)
        connection = manager["romania"].subscribers.pop(432)
        self.assertEqual(connection.name, "Dracula")
        self.assertTrue("romania" in connection.topics["subscriber"])

    def test_remove_subscriber(self):
        manager = TopicsManager()
        connection = ClientConnection(None, name="Dracula")
        manager.add_subscriber("romania", connection, 95)
        self.assertEqual(len(manager["romania"].subscribers), 1)
        self.assertTrue("romania" in connection.topics["subscriber"])
        manager.remove_subscriber("romania", 95)
        self.assertEqual(len(manager["romania"].subscribers), 0)
        self.assertFalse("romania" in connection.topics["subscriber"])

    def test_remove_subscriber_inexistent_connection(self):
        manager = TopicsManager()
        answer = manager.remove_subscriber("inexistent", None)
        self.assertIsNone(answer)

    def test_add_publisher(self):
        manager = TopicsManager()
        connection = ClientConnection(None, name="Frankenstein")
        manager.add_publisher("gernsheim", connection, 123)
        connection = manager["gernsheim"].publishers.pop(123)
        self.assertEqual(connection.name, "Frankenstein")
        self.assertTrue("gernsheim" in connection.topics["publisher"])

    def test_remove_publisher(self):
        manager = TopicsManager()
        connection = ClientConnection(None, name="Frankenstein")
        manager.add_publisher("gernsheim", connection, 123)
        self.assertEqual(len(manager["gernsheim"].publishers), 1)
        self.assertTrue("gernsheim" in connection.topics["publisher"])
        manager.remove_publisher("gernsheim", 123)
        self.assertEqual(len(manager["gernsheim"].publishers), 0)
        self.assertFalse("gernsheim" in connection.topics["publisher"])

    def test_remove_publisher_inexistent_connection(self):
        manager = TopicsManager()
        answer = manager.remove_publisher("inexistent", None)
        self.assertIsNone(answer)

    def test_remove_connection(self):
        manager = TopicsManager()
        connection = ClientConnection(None, name="Drakenstein")
        manager.add_publisher("gernsheim", connection)
        self.assertEqual(len(manager["gernsheim"].publishers), 1)
        self.assertTrue("gernsheim" in connection.topics["publisher"])
        manager.add_subscriber("romania", connection)
        self.assertEqual(len(manager["romania"].subscribers), 1)
        self.assertTrue("romania" in connection.topics["subscriber"])
        manager.remove_connection(connection)
        self.assertEqual(len(manager["romania"].subscribers), 0)
        self.assertEqual(len(manager["gernsheim"].publishers), 0)

    @patch("tornwamp.session.create_global_id", side_effect=[1, 2])
    @patch("tornwamp.topic.create_global_id", side_effect=[3, 4])
    def test_dict(self, mock_id, mock_id_2):
        manager = TopicsManager()
        mr_hyde = ClientConnection(None, name="Mr Hyde")
        mr_hyde.last_update = None
        dr_jekyll = ClientConnection(None, name="Dr Jekyll")
        dr_jekyll.last_update = None
        manager.add_subscriber("scotland", mr_hyde)
        manager.add_publisher("scotland", dr_jekyll)
        expected_dict = {
            'scotland': {
                'name': 'scotland',
                'publishers': {
                    4: {
                        'id': 2,
                        'last_update': None,
                        'name': 'Dr Jekyll',
                        'topics': {
                            'subscriber': {},
                            'publisher': {
                                'scotland': 4
                            }
                        },
                        'zombie': False,
                        'zombification_datetime': None
                    }
                },
                'subscribers': {
                    3: {
                        'id': 1,
                        'last_update': None,
                        'name': 'Mr Hyde',
                        'topics': {
                            'subscriber': {
                                'scotland': 3
                            },
                            'publisher': {}
                        },
                        'zombie': False,
                        'zombification_datetime': None
                    }
                }
            }
        }
        self.assertEqual(manager.dict, expected_dict)

    def test_get_connection(self):
        manager = TopicsManager()
        frodo = ClientConnection(None, name="Frodo")
        sam = ClientConnection(None, name="Sam")
        manager.add_subscriber("lord.of.the.rings", frodo, subscription_id=1)
        manager.add_publisher("lord.of.the.rings", sam, subscription_id=2)
        hopefully_frodo = manager.get_connection("lord.of.the.rings", 1)
        hopefully_sam = manager.get_connection("lord.of.the.rings", 2)
        self.assertEqual(frodo, hopefully_frodo)
        self.assertEqual(sam, hopefully_sam)



"""
Used to handle PubSub topics publishers and subscribers
"""
from tornwamp.identifier import create_global_id


class TopicsManager(dict):
    """
    Manages all existing topics to which connections can potentially
    publish and/or subscribe to.
    """

    def add_subscriber(self, topic_name, connection, subscription_id=None):
        """
        Add a connection as a topic's subscriber.
        """
        topic = self.get(topic_name, Topic(topic_name))
        subscription_id = subscription_id or create_global_id()
        topic.subscribers[subscription_id] = connection
        self[topic_name] = topic
        connection.add_subscription_channel(subscription_id, topic_name)
        return subscription_id

    def remove_subscriber(self, topic_name, subscription_id):
        """
        Remove a connection a topic's subscriber provided:
        - topic_name
        - subscription_id
        """
        topic = self.get(topic_name)
        if topic and subscription_id in topic.subscribers:
            connection = topic.subscribers.pop(subscription_id)
            connection.remove_subscription_channel(topic_name)

    def add_publisher(self, topic_name, connection, subscription_id=None):
        """
        Add a connection as a topic's publisher.
        """
        topic = self.get(topic_name, Topic(topic_name))
        subscription_id = subscription_id or create_global_id()
        topic.publishers[subscription_id] = connection
        self[topic_name] = topic
        connection.add_publishing_channel(subscription_id, topic_name)
        return subscription_id

    def remove_publisher(self, topic_name, subscription_id):
        """
        Remove a connection a topic's subscriber
        """
        topic = self.get(topic_name)
        if topic and subscription_id in topic.publishers:
            connection = topic.publishers.pop(subscription_id)
            connection.remove_publishing_channel(topic_name)

    def remove_connection(self, connection):
        """
        Connection is to be removed, scrap all connection publishers/subscribers in every topic
        """
        for topic_name, subscription_id in connection.topics.get("publisher", {}).items():
            topic = self.get(topic_name)
            topic.publishers.pop(subscription_id, None)

        for topic_name, subscription_id in connection.topics.get("subscriber", {}).items():
            topic = self.get(topic_name)
            topic.subscribers.pop(subscription_id, None)

    def get_connection(self, topic_name, subscription_id):
        """
        Get topic connection provided topic_name and subscription_id. Try to find
        it in subscribers, otherwise, fetches from publishers.
        Return None if it is not available in any.
        """
        topic = self[topic_name]
        return topic.subscribers.get(subscription_id) or topic.publishers.get(subscription_id)

    @property
    def dict(self):
        """
        Return a dict that is jsonifiable.
        """
        return {k: topic.dict for k, topic in self.items()}


topics = TopicsManager()


class Topic(object):
    """
    Represent a topic, containing its name, subscribers and publishers.
    """
    def __init__(self, name):
        self.name = name
        self.subscribers = {}
        self.publishers = {}

    @property
    def connections(self):
        """
        Return a set of topic connections - no matter if they are subscribers or publishers.
        """
        # About merging two dictionaries without changing the original one:

        # first version:
        # dict(self.subscribers, **self.publishers)
        # but it doesn't work with Python 3.5 if keyword args aren't strings

        # cool fancy Python3.5 way:
        # {**self.subscribers, **self.publishers}

        # Python 2+3 boring compatible way:
        conns = self.subscribers.copy()
        conns.update(self.publishers)
        return conns

    @property
    def dict(self):
        """
        Return a dict that is jsonifiable.
        """
        subscribers = {subscription_id: conn.dict for subscription_id, conn in self.subscribers.items()}
        publishers = {subscription_id: conn.dict for subscription_id, conn in self.publishers.items()}
        data = {
            "name": self.name,
            "subscribers": subscribers,
            "publishers": publishers
        }
        return data



import unittest

from tornwamp.messages import GoodbyeMessage, HelloMessage, Message, Code
from tornwamp.processors import Processor, GoodbyeProcessor, HelloProcessor,\
    UnhandledProcessor


class MockConnection(object):
    id = 1


connection = MockConnection()


class ProcessorTestCase(unittest.TestCase):

    def test_base_processor(self):
        with self.assertRaises(TypeError) as error:
            processor = Processor()
        msg = str(error.exception)
        expected_msg = "Can't instantiate abstract class Processor with abstract methods process"
        self.assertEqual(msg, expected_msg)

    def test_unhandled_processor(self):
        message = Message(*[456, 34, 'wamp.undefined.message'])
        processor = UnhandledProcessor(message, connection)
        details = "Unsupported message [456, 34, 'wamp.undefined.message']"
        response = processor.answer_message
        self.assertEqual(response.code, Code.ERROR)
        self.assertEqual(response.details["message"], details)
        self.assertEqual(response.uri, "wamp.unsupported.message")
        self.assertEqual(processor.must_close, False)

    def test_hello_processor(self):
        message = HelloMessage(realm="earth")
        processor = HelloProcessor(message, connection)
        response = processor.answer_message
        self.assertEqual(response.code, Code.WELCOME)

    def test_goodbye_processor(self):
        message = GoodbyeMessage(details={"message": "adios"}, reason="i.dont.like.you")
        processor = GoodbyeProcessor(message, connection)
        response = processor.answer_message
        self.assertEqual(response.code, Code.GOODBYE)
        self.assertEqual(response.details, {"message": "adios"})
        self.assertEqual(response.reason, "i.dont.like.you")
        self.assertTrue(processor.must_close)
        self.assertEqual(processor.close_code, 1000)
        self.assertEqual(processor.close_reason, "adios")



import unittest

from tornwamp import topic
from tornwamp.messages import Code, PublishMessage, SubscribeMessage
from tornwamp.processors.pubsub import customize
from tornwamp.session import ClientConnection
from tornwamp.topic import Topic


class CustomizeTestCase(unittest.TestCase):

    def setUp(self):
        topic_name = "education.first"
        self.original_topics = topic.topics
        connection = ClientConnection(None, user_id=7471)
        topic.topics.add_subscriber(topic_name, connection, 18273)

    def tearDown(self):
        topic.topics = self.original_topics

    def test_get_publish_direct_messages(self):
        msg = PublishMessage(request_id=168206, topic="education.first", kwargs={"type": "someMessage"})
        pub_id = 91537
        other_connection = ClientConnection(None, user_id=123)
        items = customize.get_publish_direct_messages(msg, pub_id, other_connection)
        self.assertEqual(len(items), 1)
        message = items[0]["message"]
        self.assertEqual(items[0]["websocket"], None)
        self.assertEqual(message.code, Code.EVENT)
        self.assertEqual(message.subscription_id, 18273)
        self.assertEqual(message.publication_id, 91537)
        self.assertEqual(message.kwargs, {"type": "someMessage"})

    def test_get_subscribe_direct_messages(self):
        msg = SubscribeMessage(request_id=9581, topic="dolphins.are.non.human.people")
        subscription_id = 637
        items = customize.get_subscribe_direct_messages(msg, subscription_id)
        self.assertEqual(items, [])



"""
This module should be overwride subscription procedures:
- authorize: if we will allow a connection to subscribe to a topic or not
- register: what we expect to happen (e.g. connection becomes a subscriber or
publisher of that topic)
"""
from tornwamp import topic as tornwamp_topic
from tornwamp.messages import EventMessage


def authorize_publication(topic_name, connection):
    """
    Says if a user can publish to a topic or not.
    Return: True or False and the error message ("" if no error occured).
    """
    assert topic_name, "authorize_publication requires topic_name"
    assert connection, "authorize_publication requires connection"
    return True, ""


def authorize_subscription(topic_name, connection):
    """
    Says if a user can subscribe to a topic or not.
    Return: True or False and the error message ("" if no error occured).
    """
    assert topic_name, "authorize_subscription requires topic_name"
    assert connection, "authorize_subscription requires connection"
    return True, ""


def add_subscriber(topic, connection):
    """
    By default, the connection is added as a subscriber of that topic.
    Return subscription ID.
    """
    subscription_id = tornwamp_topic.topics.add_subscriber(topic, connection)
    return subscription_id


def get_subscribe_direct_messages(subscribe_message, subscription_id):
    """
    Return a list of dictionaries containing websocket and what message they
    should receive. This is called from SubscribeProcessor when it succeeds.

    Sample response:
    [
        {
            "websocket": <tornwamp.WAMPHandler>,
            "message": <tornwamp.messages.Event>
        }
    ]
    """
    assert subscribe_message, "get_subscribe_direct_messages requires subscribe_message"
    assert subscription_id, "get_subscribe_direct_messages requires subscription_id"
    return []


def get_publish_direct_messages(publish_message, publication_id, publisher_connection):
    """
    Return a list of dictionaries containing websocket and what message they
    should receive. This is called from PublishProcessor when it succeeds.

    Sample response:
    [
        {
            "websocket": <tornwamp.WAMPHandler>,
            "message": <tornwamp.messages.Event>
        }
    """
    data = []
    topic_name = publish_message.topic
    topic = tornwamp_topic.topics.get(topic_name)
    if topic:
        for subscription_id, connection in topic.subscribers.items():
            if connection != publisher_connection:
                event_message = EventMessage(
                    subscription_id=subscription_id,
                    publication_id=publication_id,
                    args=publish_message.args,
                    kwargs=publish_message.kwargs,
                )
                item = {
                    "websocket": connection._websocket,
                    "message": event_message
                }
                data.append(item)
    return data



"""
This module allows customization of which methos are supported when using
RPC.
"""

from tornwamp.messages import ResultMessage


def ping(call_message, connection):
    """
    Return a answer (ResultMessage) and empty list direct_messages.
    """
    assert connection, "ping requires connection"

    answer = ResultMessage(
        request_id=call_message.request_id,
        details=call_message.details,
        args=["Ping response"]
    )
    return answer, []

procedures = {
    "ping": ping
}



import unittest
from mock import patch

from tornwamp.messages import Code, PublishMessage, SubscribeMessage
from tornwamp.processors.pubsub import PublishProcessor, SubscribeProcessor
from tornwamp.session import ClientConnection


class SubscribeProcessorTestCase(unittest.TestCase):

    def test_succeed(self):
        message = SubscribeMessage(request_id=123, topic="olympic.games")
        connection = ClientConnection(None)
        processor = SubscribeProcessor(message, connection)
        answer = processor.answer_message
        self.assertEqual(answer.code, Code.SUBSCRIBED)
        self.assertEqual(answer.request_id, 123)

    @patch("tornwamp.processors.pubsub.customize.authorize_subscription", return_value=(False, "Your problem"))
    def test_fail(self, mock_authorize):
        message = SubscribeMessage(request_id=234, topic="olympic.games")
        connection = ClientConnection(None)
        processor = SubscribeProcessor(message, connection)
        answer = processor.answer_message
        self.assertEqual(answer.code, Code.ERROR)
        self.assertEqual(answer.request_id, 234)
        self.assertEqual(answer.request_code, Code.SUBSCRIBE)
        self.assertEqual(answer.uri, "tornwamp.subscribe.unauthorized")


class PublishProcessorTestCase(unittest.TestCase):

    def test_succeed_without_acknowledge(self):
        message = PublishMessage(request_id=345, topic="world.cup")
        connection = ClientConnection(None)
        processor = PublishProcessor(message, connection)
        answer = processor.answer_message
        self.assertEqual(answer, None)

    def test_succeed_with_acknowledge(self):
        options = {"acknowledge": True}
        message = PublishMessage(request_id=345, topic="world.cup", options=options)
        connection = ClientConnection(None)
        processor = PublishProcessor(message, connection)
        answer = processor.answer_message
        self.assertEqual(answer.code, Code.PUBLISHED)
        self.assertEqual(answer.request_id, 345)

    @patch("tornwamp.processors.pubsub.customize.authorize_publication", return_value=(False, "Your problem"))
    def test_fail(self, mock_authorize):
        message = PublishMessage(request_id=456, topic="world.cup")
        connection = ClientConnection(None)
        processor = PublishProcessor(message, connection)
        answer = processor.answer_message
        self.assertEqual(answer.code, Code.ERROR)
        self.assertEqual(answer.request_id, 456)
        self.assertEqual(answer.request_code, Code.PUBLISH)
        self.assertEqual(answer.uri, "tornwamp.publish.unauthorized")



"""
WAMP messages definitions and serializers.

Compatible with WAMP Document Revision: RC3, 2014/08/25, available at:
https://github.com/tavendo/WAMP/blob/master/spec/basic.md
"""
import json
from copy import deepcopy

from enum import IntEnum
from tornwamp.identifier import create_global_id


class Code(IntEnum):
    """
    Enum which represents currently supported WAMP messages.
    """
    HELLO = 1
    WELCOME = 2
    ABORT = 3
    # CHALLENGE = 4
    # AUTHENTICATE = 5
    GOODBYE = 6
    # HEARTBEAT = 7
    ERROR = 8
    PUBLISH = 16
    PUBLISHED = 17
    SUBSCRIBE = 32
    SUBSCRIBED = 33
    UNSUBSCRIBE = 34
    UNSUBSCRIBED = 35
    EVENT = 36
    CALL = 48
    # CANCEL = 49
    RESULT = 50
    # REGISTER = 64
    # REGISTERED = 65
    # UNREGISTER = 66
    # UNREGISTERED = 67
    # INVOCATION = 68
    # INTERRUPT = 69
    # YIELD = 70


class Message(object):
    """
    Represent any WAMP message.
    """
    details = {}

    def __init__(self, code, *data, **kdata):
        self.code = code
        self.value = [code] + list(data)
        self.args = kdata.get("args", [])
        self.kwargs = kdata.get("kwargs", {})

    @property
    def id(self):
        """
        For all kinds of messages (except ERROR) that have [Request|id], it is
        in the second position of the array.
        """
        if (len(self.value) > 1) and isinstance(self.value[1], int):
            return self.value[1]
        return -1

    @property
    def json(self):
        """
        Create a JSON representation of this message.
        """
        message_value = deepcopy(self.value)
        # TODO: test
        for index, item in enumerate(message_value):
            if isinstance(item, Code):
                message_value[index] = item.value
        return json.dumps(message_value)

    def error(self, text, info=None):
        """
        Add error description and aditional information.

        This is useful for ABORT and ERROR messages.
        """
        self.details["message"] = text
        if info:
            self.details["details"] = info

    @classmethod
    def from_text(cls, text):
        """
        Decode text to JSON and return a Message object accordingly.
        """
        raw = json.loads(text)
        raw[0] = Code(raw[0])  # make it an object of type Code
        return cls(*raw)

    def _update_args_and_kargs(self):
        """
        Append args and kwargs to message value, according to their existance
        or not.
        """
        if self.kwargs:
            self.value.append(self.args)
            self.value.append(self.kwargs)
        else:
            if self.args:
                self.value.append(self.args)


class HelloMessage(Message):
    """
    Sent by a Client to initiate opening of a WAMP session:
    [HELLO, Realm|uri, Details|dict]

    https://github.com/tavendo/WAMP/blob/master/spec/basic.md#hello
    """

    def __init__(self, code=Code.HELLO, realm="", details=None):
        self.code = code
        self.realm = realm
        self.details = details if details else {}
        self.value = [self.code, self.realm, self.details]


class AbortMessage(Message):
    """
    Both the Router and the Client may abort the opening of a WAMP session
    [ABORT, Details|dict, Reason|uri]

    https://github.com/tavendo/WAMP/blob/master/spec/basic.md#abort
    """

    def __init__(self, code=Code.ABORT, details=None, reason=None):
        assert reason is not None, "AbortMessage must have a reason"
        self.code = code
        self.details = details or {}
        self.reason = reason
        self.value = [self.code, self.details, self.reason]


DEFAULT_WELCOME_DETAILS = {
    "authrole": "anonymous",
    "authmethod": "anonymous",
    "roles": {
        "broker": {
            "features": {
                "publisher_identification": True,
                "publisher_exclusion": True,
                "subscriber_blackwhite_listing": True
            }
        },
        "dealer": {
            "features": {
                "progressive_call_results": True,
                "caller_identification": True
            }
        }
    },
    "authid": "jiQHbkkOxD1EFI7mJ1JITy3K"
}


class WelcomeMessage(Message):
    """
    Sent from the server side to open a WAMP session.
    The WELCOME is a reply message to the Client's HELLO.

    [WELCOME, Session|id, Details|dict]

    https://github.com/tavendo/WAMP/blob/master/spec/basic.md#welcome
    """

    def __init__(self, code=Code.WELCOME, session_id=None, details=None):
        self.code = code
        self.session_id = session_id or create_global_id()
        self.details = details or DEFAULT_WELCOME_DETAILS
        self.value = [self.code, self.session_id, self.details]


class GoodbyeMessage(Message):
    """
    Both the Server and the Client may abort the opening of a WAMP session
    [ABORT, Details|dict, Reason|uri]
    """

    def __init__(self, code=Code.GOODBYE, details=None, reason=None):
        self.code = code
        self.details = details or {}
        self.reason = reason or ""
        self.value = [self.code, self.details, self.reason]


class ResultMessage(Message):
    """
    Result of a call as returned by Dealer to Caller.

    [RESULT, CALL.Request|id, Details|dict]
    [RESULT, CALL.Request|id, Details|dict, YIELD.Arguments|list]
    [RESULT, CALL.Request|id, Details|dict, YIELD.Arguments|list, YIELD.ArgumentsKw|dict]
    """

    def __init__(self, code=Code.RESULT, request_id=None, details=None, args=None, kwargs=None):
        assert request_id is not None, "ResultMessage must have request_id"
        self.code = code
        self.request_id = request_id
        self.details = details or {}
        self.args = args or []
        self.kwargs = kwargs or {}
        self.value = [
            self.code,
            self.request_id,
            self.details
        ]
        self._update_args_and_kargs()


class CallMessage(Message):
    """
    Call as originally issued by the Caller to the Dealer.

    [CALL, Request|id, Options|dict, Procedure|uri]
    [CALL, Request|id, Options|dict, Procedure|uri, Arguments|list]
    [CALL, Request|id, Options|dict, Procedure|uri, Arguments|list, ArgumentsKw|dict]
    """

    def __init__(self, code=Code.CALL, request_id=None, options=None, procedure=None, args=None, kwargs=None):
        assert request_id is not None, "CallMessage must have request_id"
        assert procedure is not None, "CallMessage must have procedure"
        self.code = code
        self.request_id = request_id
        self.options = options or {}
        self.procedure = procedure
        self.args = args or []
        self.kwargs = kwargs or {}
        self.value = [
            self.code,
            self.request_id,
            self.options,
            self.procedure,
        ]
        self._update_args_and_kargs()


class ErrorMessage(Message):
    """
    Error reply sent by a Peer as an error response to different kinds of
    requests.

    [ERROR, REQUEST.Type|int, REQUEST.Request|id, Details|dict, Error|uri]
    [ERROR, REQUEST.Type|int, REQUEST.Request|id, Details|dict, Error|uri,
        Arguments|list]
    [ERROR, REQUEST.Type|int, REQUEST.Request|id, Details|dict, Error|uri,
        Arguments|list, ArgumentsKw|dict]
    """

    def __init__(self, code=Code.ERROR, request_code=None, request_id=None, details=None, uri=None, args=None, kwargs=None):
        assert request_code is not None, "ErrorMessage must have request_code"
        assert request_id is not None, "ErrorMessage must have request_id"
        assert uri is not None, "ErrorMessage must have uri"
        self.code = code
        self.request_code = request_code
        self.request_id = request_id
        self.details = details or {}
        self.uri = uri
        self.args = args or []
        self.kwargs = kwargs or {}
        self.value = [
            self.code,
            self.request_code,
            self.request_id,
            self.details,
            self.uri
        ]
        self._update_args_and_kargs()


class SubscribeMessage(Message):
    """
    A Subscriber communicates its interest in a topic to the Server by sending
    a SUBSCRIBE message:
    [SUBSCRIBE, Request|id, Options|dict, Topic|uri]
    """

    def __init__(self, code=Code.SUBSCRIBE, request_id=None, options=None, topic=None):
        assert request_id is not None, "SubscribeMessage must have request_id"
        assert topic is not None, "SubscribeMessage must have topic"
        self.code = code
        self.request_id = request_id
        self.options = options or {}
        self.topic = topic
        self.value = [self.code, self.request_id, self.options, self.topic]


class SubscribedMessage(Message):
    """
    If the Broker is able to fulfill and allow the subscription, it answers by
    sending a SUBSCRIBED message to the Subscriber:
    [SUBSCRIBED, SUBSCRIBE.Request|id, Subscription|id]
    """
    def __init__(self, code=Code.SUBSCRIBED, request_id=None, subscription_id=None):
        assert request_id is not None, "SubscribedMessage must have request_id"
        assert subscription_id is not None, "SubscribedMessage must have subscription_id"
        self.code = code
        self.request_id = request_id
        self.subscription_id = subscription_id
        self.value = [self.code, self.request_id, self.subscription_id]


class PublishMessage(Message):
    """
    Sent by a Publisher to a Broker to publish an event.

    [PUBLISH, Request|id, Options|dict, Topic|uri]
    [PUBLISH, Request|id, Options|dict, Topic|uri, Arguments|list]
    [PUBLISH, Request|id, Options|dict, Topic|uri, Arguments|list, ArgumentsKw|dict]
    """
    def __init__(self, code=Code.PUBLISH, request_id=None, options=None, topic=None, args=None, kwargs=None):
        assert request_id is not None, "PublishMessage must have request_id"
        assert topic is not None, "PublishMessage must have topic"
        self.code = code
        self.request_id = request_id
        self.options = options or {}
        self.topic = topic
        self.args = args or []
        self.kwargs = kwargs or {}
        self.value = [self.code, self.request_id, self.options, self.topic]
        self._update_args_and_kargs()


class PublishedMessage(Message):
    """
    Acknowledge sent by a Broker to a Publisher for acknowledged publications.

    [PUBLISHED, PUBLISH.Request|id, Publication|id]
    """
    def __init__(self, code=Code.PUBLISHED, request_id=None, publication_id=None):
        assert request_id is not None, "PublishedMessage must have request_id"
        assert publication_id is not None, "PublishedMessage must have publication_id"
        self.code = code
        self.request_id = request_id
        self.publication_id = publication_id
        self.value = [self.code, self.request_id, self.publication_id]


class EventMessage(Message):
    """
    Event dispatched by Broker to Subscribers for subscription the event was matching.

    [EVENT, SUBSCRIBED.Subscription|id, PUBLISHED.Publication|id, Details|dict]
    [EVENT, SUBSCRIBED.Subscription|id, PUBLISHED.Publication|id, Details|dict, PUBLISH.Arguments|list]
    [EVENT, SUBSCRIBED.Subscription|id, PUBLISHED.Publication|id, Details|dict, PUBLISH.Arguments|list, PUBLISH.ArgumentsKw|dict]
    """
    def __init__(self, code=Code.EVENT, subscription_id=None, publication_id=None, details=None, args=None, kwargs=None):
        assert subscription_id is not None, "EventMessage must have subscription_id"
        assert publication_id is not None, "EventMessage must have publication_id"
        self.code = code
        self.subscription_id = subscription_id
        self.publication_id = publication_id
        self.details = details or {}
        self.args = args or []
        self.kwargs = kwargs or {}
        self.value = [self.code, self.subscription_id, self.publication_id, self.details]
        self._update_args_and_kargs()


class UnsubscribeMessage(Message):
    """
    Unsubscribe request sent by a Subscriber to a Broker to unsubscribe a subscription.
    [UNSUBSCRIBE, Request|id, SUBSCRIBED.Subscription|id]
    """
    def __init__(self, code=Code.UNSUBSCRIBE, request_id=None, subscription_id=None):
        assert request_id is not None, "UnsubscribeMessage must have request_id"
        assert subscription_id is not None, "UnsubscribeMessage must have subscription_id"
        self.code = code
        self.request_id = request_id
        self.subscription_id = subscription_id
        self.value = [self.code, self.request_id, self.subscription_id]


class UnsubscribedMessage(Message):
    """
    Acknowledge sent by a Broker to a Subscriber to acknowledge unsubscription.
    [UNSUBSCRIBED, UNSUBSCRIBE.Request|id]
    """
    def __init__(self, code=Code.UNSUBSCRIBED, request_id=None):
        assert request_id is not None, "UnsubscribedMessage must have request_id"
        self.code = code
        self.request_id = request_id
        self.value = [self.code, self.request_id]


CODE_TO_CLASS = {
    Code.HELLO: HelloMessage,
    Code.WELCOME: WelcomeMessage,
    Code.ABORT: AbortMessage,
    # CHALLENGE = 4
    # AUTHENTICATE = 5
    Code.GOODBYE: GoodbyeMessage,
    # HEARTBEAT = 7
    Code.ERROR: ErrorMessage,
    Code.PUBLISH: PublishMessage,
    Code.PUBLISHED: PublishedMessage,
    Code.SUBSCRIBE: SubscribeMessage,
    Code.SUBSCRIBED: SubscribedMessage,
    Code.UNSUBSCRIBE: UnsubscribeMessage,
    Code.UNSUBSCRIBED: UnsubscribedMessage,
    Code.EVENT: EventMessage,
    Code.CALL: CallMessage,
    # CANCEL = 49
    Code.RESULT: ResultMessage
    # REGISTER = 64
    # REGISTERED = 65
    # UNREGISTER = 66
    # UNREGISTERED = 67
    # INVOCATION = 68
    # INTERRUPT = 69
    # YIELD = 70
}

ERROR_PRONE_CODES = [Code.CALL, Code.SUBSCRIBE, Code.UNSUBSCRIBE, Code.PUBLISH]


def build_error_message(in_message, uri, description):
    """
    Return ErrorMessage instance (*) provided:
    - incoming message which generated error
    - error uri
    - error description

    (*) If incoming message is not prone to ERROR message reponse, return None.
    """
    msg = Message.from_text(in_message)
    if msg.code in ERROR_PRONE_CODES:
        MsgClass = CODE_TO_CLASS[msg.code]
        msg = MsgClass.from_text(in_message)
        answer = ErrorMessage(
            request_code=msg.code,
            request_id=msg.request_id,
            uri=uri
        )
        answer.error(description)
        return answer.json



"""
Compatible with WAMP Document Revision: RC3, 2014/08/25.

WAMP Identifiers definition is available at:
https://github.com/tavendo/WAMP/blob/master/spec/basic.md#ids


{citation}

WAMP needs to identify the following ephemeral entities each in the scope noted

- Sessions (global scope)
- Publications (global scope)
- Subscriptions (router scope)
- Registrations (router scope)
- Requests (session scope)

These are identified in WAMP using IDs that are integers between (inclusive) 0
and 2^53 (9007199254740992):

IDs in the global scope MUST be drawn randomly from a uniform distribution over
the complete range [0, 2^53]

IDs in the router fscope can be chosen freely by the specific router
implementation

IDs in the session scope SHOULD be incremented by 1 beginning with 1 (for each
direction - Client-to-Router and Router-to-Client)

{citation}
"""
import random

MIN_ID = 0
MAX_ID = 2 ** 53

existing_ids = []


def create_global_id():
    """
    Return a global scope ID, which is not in existing_ids list provided.
    This function also appends new ID to original existing_ids list.

    According to WAMP specification:
    "IDs in the global scope MUST be drawn randomly from a uniform distribution
    over the complete range [0, 2^53]"
    """
    new_id = None
    while new_id not in existing_ids:
        candidate_id = random.randint(MIN_ID, MAX_ID)
        if candidate_id not in existing_ids:
            new_id = candidate_id
            existing_ids.append(new_id)
    return new_id



import unittest

from tornwamp import messages as wamp
from tornwamp.messages import Code
from tornwamp.identifier import MIN_ID, MAX_ID


class MessageTestCase(unittest.TestCase):

    def test_base_message(self):
        raw = [Code.HELLO, "somerealm", {}]
        message = wamp.Message(*raw)
        self.assertEqual(message.code, Code.HELLO)
        self.assertEqual(message.value, raw)
        self.assertEqual(message.json, '[1, "somerealm", {}]')

    def test_message_without_id(self):
        raw = [201]
        message = wamp.Message(*raw)
        self.assertEqual(message.id, -1)

    def test_from_text(self):
        msg = wamp.Message.from_text("[1]")
        self.assertTrue(isinstance(msg, wamp.Message))
        msg = wamp.HelloMessage.from_text("[1]")
        self.assertTrue(isinstance(msg, wamp.HelloMessage))

    def test_error(self):
        msg = wamp.Message(34)
        msg.error("Some error", {"answer": 42})
        expected_details = {
            "message": "Some error",
            "details": {"answer": 42}
        }
        self.assertEqual(msg.details, expected_details)

    def test_hello_message(self):
        hello_message = wamp.HelloMessage(realm="world", details={"roles": {}})
        self.assertEqual(hello_message.code, Code.HELLO)
        self.assertEqual(hello_message.realm, "world")
        self.assertEqual(hello_message.details, {"roles": {}})
        self.assertEqual(hello_message.json, '[1, "world", {"roles": {}}]')

    def test_abort_message(self):
        abort_message = wamp.AbortMessage(reason="user.unauthorized")
        abort_message.error("Invalid cookie")
        self.assertEqual(abort_message.code, Code.ABORT)
        self.assertEqual(abort_message.details, {"message": "Invalid cookie"})
        self.assertEqual(abort_message.reason, "user.unauthorized")
        expected = '[3, {"message": "Invalid cookie"}, "user.unauthorized"]'
        self.assertEqual(abort_message.json, expected)

    def test_abort_message_without_reason(self):
        with self.assertRaises(AssertionError) as error:
            abort_message = wamp.AbortMessage()
        computed = str(error.exception)
        expected = "AbortMessage must have a reason"
        self.assertEqual(computed, expected)

    def test_welcome_message(self):
        abort_message = wamp.WelcomeMessage()
        self.assertEqual(abort_message.code, Code.WELCOME)
        self.assertEqual(abort_message.details, wamp.DEFAULT_WELCOME_DETAILS)
        self.assertTrue(MIN_ID < abort_message.session_id < MAX_ID)

    def test_error_message_simple(self):
        error_message = wamp.ErrorMessage(
            request_code=2,
            request_id=3126,
            uri="some.very.buggy.error"
        )
        self.assertEqual(error_message.code, Code.ERROR)
        self.assertEqual(error_message.details, {})
        expected = [8, 2, 3126, {}, "some.very.buggy.error"]
        self.assertEqual(error_message.value, expected)

    def test_error_message_missing_request_code(self):
        with self.assertRaises(AssertionError) as error:
            abort_message = wamp.ErrorMessage()
        computed = str(error.exception)
        expected = "ErrorMessage must have request_code"
        self.assertEqual(computed, expected)

    def test_error_message_kwargs_only(self):
        error_message = wamp.ErrorMessage(
            request_code=3,
            request_id=7432,
            uri="horrible.exception",
            kwargs={'a': 1}
        )
        self.assertEqual(error_message.code, Code.ERROR)
        self.assertEqual(error_message.details, {})
        expected = [8, 3, 7432, {}, "horrible.exception", [], {'a': 1}]
        self.assertEqual(error_message.value, expected)

    def test_error_message_args_only(self):
        error_message = wamp.ErrorMessage(
            request_code=4,
            request_id=8259,
            uri="light.bug",
            args=["banana"]
        )
        self.assertEqual(error_message.code, Code.ERROR)
        self.assertEqual(error_message.details, {})
        expected = [8, 4, 8259, {}, "light.bug", ["banana"]]
        self.assertEqual(error_message.value, expected)

    def test_subscribe_message(self):
        subscribe_message = wamp.SubscribeMessage(request_id=1395, topic="lesson.1")
        self.assertEqual(subscribe_message.code, Code.SUBSCRIBE)
        self.assertEqual(subscribe_message.request_id, 1395)
        self.assertEqual(subscribe_message.options, {})
        self.assertEqual(subscribe_message.topic, "lesson.1")
        expected = [32, 1395, {}, "lesson.1"]
        self.assertEqual(subscribe_message.value, expected)

    def test_subscribed_message(self):
        subscribed_message = wamp.SubscribedMessage(request_id=4872, subscription_id=653)
        self.assertEqual(subscribed_message.code, Code.SUBSCRIBED)
        self.assertEqual(subscribed_message.request_id, 4872)
        self.assertEqual(subscribed_message.subscription_id, 653)
        expected = [33, 4872, 653]
        self.assertEqual(subscribed_message.value, expected)

    def test_publish_message(self):
        publish_message = wamp.PublishMessage(request_id=514, topic="zazie")
        self.assertEqual(publish_message.code, Code.PUBLISH)
        self.assertEqual(publish_message.request_id, 514)
        self.assertEqual(publish_message.topic, "zazie")
        expected = [16, 514, {}, "zazie"]
        self.assertEqual(publish_message.value, expected)

    def test_published_message(self):
        published_message = wamp.PublishedMessage(request_id=681, publication_id=5092)
        self.assertEqual(published_message.code, Code.PUBLISHED)
        self.assertEqual(published_message.request_id, 681)
        self.assertEqual(published_message.publication_id, 5092)
        expected = [17, 681, 5092]
        self.assertEqual(published_message.value, expected)

    def test_event_message(self):
        event_message = wamp.EventMessage(subscription_id=74, publication_id=27)
        self.assertEqual(event_message.code, Code.EVENT)
        self.assertEqual(event_message.subscription_id, 74)
        self.assertEqual(event_message.publication_id, 27)
        expected = [36, 74, 27, {}]
        self.assertEqual(event_message.value, expected)



import unittest

from tornwamp import messages as wamp
from tornwamp.messages import Code
from tornwamp.identifier import MIN_ID, MAX_ID


class MessageTestCase(unittest.TestCase):

    def test_base_message(self):
        raw = [Code.HELLO, "somerealm", {}]
        message = wamp.Message(*raw)
        self.assertEqual(message.code, Code.HELLO)
        self.assertEqual(message.value, raw)
        self.assertEqual(message.json, '[1, "somerealm", {}]')

    def test_message_without_id(self):
        raw = [201]
        message = wamp.Message(*raw)
        self.assertEqual(message.id, -1)

    def test_from_text(self):
        msg = wamp.Message.from_text("[1]")
        self.assertTrue(isinstance(msg, wamp.Message))
        msg = wamp.HelloMessage.from_text("[1]")
        self.assertTrue(isinstance(msg, wamp.HelloMessage))

    def test_error(self):
        msg = wamp.Message(34)
        msg.error("Some error", {"answer": 42})
        expected_details = {
            "message": "Some error",
            "details": {"answer": 42}
        }
        self.assertEqual(msg.details, expected_details)

    def test_hello_message(self):
        hello_message = wamp.HelloMessage(realm="world", details={"roles": {}})
        self.assertEqual(hello_message.code, Code.HELLO)
        self.assertEqual(hello_message.realm, "world")
        self.assertEqual(hello_message.details, {"roles": {}})
        self.assertEqual(hello_message.json, '[1, "world", {"roles": {}}]')

    def test_abort_message(self):
        abort_message = wamp.AbortMessage(reason="user.unauthorized")
        abort_message.error("Invalid cookie")
        self.assertEqual(abort_message.code, Code.ABORT)
        self.assertEqual(abort_message.details, {"message": "Invalid cookie"})
        self.assertEqual(abort_message.reason, "user.unauthorized")
        expected = '[3, {"message": "Invalid cookie"}, "user.unauthorized"]'
        self.assertEqual(abort_message.json, expected)

    def test_abort_message_without_reason(self):
        with self.assertRaises(AssertionError) as error:
            abort_message = wamp.AbortMessage()
        computed = str(error.exception)
        expected = "AbortMessage must have a reason"
        self.assertEqual(computed, expected)

    def test_welcome_message(self):
        abort_message = wamp.WelcomeMessage()
        self.assertEqual(abort_message.code, Code.WELCOME)
        self.assertEqual(abort_message.details, wamp.DEFAULT_WELCOME_DETAILS)
        self.assertTrue(MIN_ID < abort_message.session_id < MAX_ID)

    def test_error_message_simple(self):
        error_message = wamp.ErrorMessage(
            request_code=2,
            request_id=3126,
            uri="some.very.buggy.error"
        )
        self.assertEqual(error_message.code, Code.ERROR)
        self.assertEqual(error_message.details, {})
        expected = [8, 2, 3126, {}, "some.very.buggy.error"]
        self.assertEqual(error_message.value, expected)

    def test_error_message_missing_request_code(self):
        with self.assertRaises(AssertionError) as error:
            abort_message = wamp.ErrorMessage()
        computed = str(error.exception)
        expected = "ErrorMessage must have request_code"
        self.assertEqual(computed, expected)

    def test_error_message_kwargs_only(self):
        error_message = wamp.ErrorMessage(
            request_code=3,
            request_id=7432,
            uri="horrible.exception",
            kwargs={'a': 1}
        )
        self.assertEqual(error_message.code, Code.ERROR)
        self.assertEqual(error_message.details, {})
        expected = [8, 3, 7432, {}, "horrible.exception", [], {'a': 1}]
        self.assertEqual(error_message.value, expected)

    def test_error_message_args_only(self):
        error_message = wamp.ErrorMessage(
            request_code=4,
            request_id=8259,
            uri="light.bug",
            args=["banana"]
        )
        self.assertEqual(error_message.code, Code.ERROR)
        self.assertEqual(error_message.details, {})
        expected = [8, 4, 8259, {}, "light.bug", ["banana"]]
        self.assertEqual(error_message.value, expected)

    def test_result_message(self):
        result_message = wamp.ResultMessage(request_id=8259)
        self.assertEqual(result_message.code, Code.RESULT)
        self.assertEqual(result_message.details, {})
        self.assertEqual(result_message.args, [])
        self.assertEqual(result_message.kwargs, {})
        expected = [50, 8259, {}]
        self.assertEqual(result_message.value, expected)

    def test_unsubscribe_message(self):
        unsubscribe_message = wamp.UnsubscribeMessage(request_id=1409, subscription_id=2)
        self.assertEqual(unsubscribe_message.code, Code.UNSUBSCRIBE)
        self.assertEqual(unsubscribe_message.request_id, 1409)
        self.assertEqual(unsubscribe_message.subscription_id, 2)
        expected = [34, 1409, 2]
        self.assertEqual(unsubscribe_message.value, expected)

    def test_unsubscribed_message(self):
        unsubscribed_message = wamp.UnsubscribedMessage(request_id=723)
        self.assertEqual(unsubscribed_message.code, Code.UNSUBSCRIBED)
        self.assertEqual(unsubscribed_message.request_id, 723)
        expected = [35, 723]
        self.assertEqual(unsubscribed_message.value, expected)



import unittest

from tornwamp.messages import CallMessage, ResultMessage, Code
from tornwamp.processors.rpc import CallProcessor


class MockConnection(object):
    id = 1


connection = MockConnection()


class CallProcessorTestCase(unittest.TestCase):

    def test_call_processor_ping(self):
        message = CallMessage(request_id=918273, procedure="ping")
        processor = CallProcessor(message, connection)
        response = processor.answer_message
        self.assertEqual(response.code, Code.RESULT)
        self.assertEqual(response.request_id, 918273)
        self.assertEqual(response.args[0], "Ping response")

    def test_call_processor_unsupported_method(self):
        message = CallMessage(request_id=192837, procedure="abc")
        processor = CallProcessor(message, connection)
        response = processor.answer_message
        self.assertEqual(response.code, Code.ERROR)
        self.assertEqual(response.request_id, 192837)
        self.assertEqual(response.uri, 'wamp.rpc.unsupported.procedure')
        self.assertEqual(response.details["message"], "The procedure abc doesn't exist")



"""
RPC processors.

Compatible with WAMP Document Revision: RC3, 2014/08/25, available at:
https://github.com/tavendo/WAMP/blob/master/spec/basic.md
"""

from tornwamp.messages import CallMessage, ResultMessage, ErrorMessage
from tornwamp.processors import Processor
from tornwamp.processors.rpc import customize


class CallProcessor(Processor):
    """
    Responsible for dealing with CALL messages.
    """

    def process(self):
        """
        Call method defined in tornwamp.customize.procedures (dict).

        Each method should return:
        - RESPONSE
        - ERROR

        Which will be the processor's answer message.'
        """
        msg = CallMessage(*self.message.value)
        direct_messages = []
        method_name = msg.procedure
        if (method_name in customize.procedures):
            method = customize.procedures[method_name]
            answer, direct_messages = method(*msg.args, call_message=msg, connection=self.connection, **msg.kwargs)
        else:
            error_uri = "wamp.rpc.unsupported.procedure"
            error_msg = "The procedure {} doesn't exist".format(method_name)
            response_msg = ErrorMessage(
                request_code=msg.code,
                request_id=msg.request_id,
                details={"call": msg.json},
                uri=error_uri
            )
            response_msg.error(error_msg)
            answer = response_msg
        self.answer_message = answer
        self.direct_messages = direct_messages



from setuptools import setup, find_packages


README = open('README.rst').read()


setup(name="tornwamp",
      author="Tatiana Al-Chueyr Martins",
      author_email="tatiana.alchueyr@gmail.com",
      classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5'
        ],
      download_url = 'http://pypi.python.org/pypi/tornwamp',
      description=u"WAMP (Web Application Messaging Protocol) utilities",
      include_package_data=True,
      install_requires=["tornado>=4.0", "enum34"],
      license="Apache License",
      long_description=README,
      packages=find_packages(),
      tests_require=["coverage==4.0.3", "nose==1.3.7", "pep8==1.7.0", "mock==1.0.1", "pylint==1.5.4"],
      url = "http://github.com/ef-ctx/tornwamp",
      version="1.1.0"
)



"""
WAMP-PubSub processors.
"""
from tornwamp.identifier import create_global_id
from tornwamp.messages import ErrorMessage, EventMessage, PublishMessage, PublishedMessage, SubscribeMessage, SubscribedMessage
from tornwamp.processors import Processor
from tornwamp.processors.pubsub import customize


class SubscribeProcessor(Processor):
    """
    Responsible for dealing SUBSCRIBE messages.
    """
    def process(self):
        """
        Return SUBSCRIBE message based on the input HELLO message.
        """
        received_message = SubscribeMessage(*self.message.value)
        allow, msg = customize.authorize_subscription(received_message.topic, self.connection)
        if allow:
            subscription_id = customize.add_subscriber(received_message.topic, self.connection)
            answer = SubscribedMessage(
                request_id=received_message.request_id,
                subscription_id=subscription_id
            )
            self.direct_messages = customize.get_subscribe_direct_messages(received_message, subscription_id)
        else:
            answer = ErrorMessage(
                request_id=received_message.request_id,
                request_code=received_message.code,
                uri="tornwamp.subscribe.unauthorized"
            )
            answer.error(msg)
        self.answer_message = answer


class PublishProcessor(Processor):
    """
    Responsible for dealing SUBSCRIBE messages.
    """
    def process(self):
        """
        Return SUBSCRIBE message based on the input HELLO message.
        """
        received_message = PublishMessage(*self.message.value)
        allow, msg = customize.authorize_publication(received_message.topic, self.connection)
        answer = None
        if allow:
            publication_id = create_global_id()
            if received_message.options.get("acknowledge"):
                answer = PublishedMessage(
                    request_id=received_message.request_id,
                    publication_id=publication_id
                )
            self.direct_messages = customize.get_publish_direct_messages(
                received_message,
                publication_id,
                self.connection
            )
        else:
            answer = ErrorMessage(
                request_id=received_message.request_id,
                request_code=received_message.code,
                uri="tornwamp.publish.unauthorized"
            )
            answer.error(msg)
        self.answer_message = answer
